<?php
/*
Plugin Name: guest-comments
Description: A plugin to allow logged-in users to comment as guests (ie: to provide name/email/website)
Version: 0.3.3
Author: Joseph Roberts (RobertsDev Unlimited)
Author URI: www.robertsdev.com
License: GPLv3
*/

/*
	This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranties of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    To read the GNU General Public License, see http://www.gnu.org/licenses.
*/


class guest_comments
{
	function init()
	{
		// Register action hooks to expose UI to set guest_comments_enabled yes/no setting on user profile pages
		add_action('show_user_profile', array( $this, 'render_user_profile_setting' ) );		// current user
		add_action('edit_user_profile', array( $this, 'render_user_profile_setting' ) );		// all users
		
		// Register action hooks to save guest_comments_enabled yes/no setting to user metadata
		add_action('personal_options_update',  array( $this, 'save_user_profile_setting' ) );	// current user
		add_action('edit_user_profile_update', array( $this, 'save_user_profile_setting' ) );	// all users
		
		// Register filter hooks to allow logged-in users with guest_comments_enabled setting to comment as guests
		add_filter( 'comment_form_logged_in', array( $this, 'my_comment_form_logged_in' ), 10, 3 );		// as comment form is being rendered
		add_filter( 'preprocess_comment',     array( $this, 'my_preprocess_comment' ) );				// just before comments are saved
		
		add_filter( 'comment_form_default_fields', array( $this, 'remove_url_field' ), 100, 1 );		// 100 to force it to run after others

		// add the admin options page
		add_action('admin_init', array( $this, 'plugin_admin_init') );
		add_action('admin_menu', array( $this, 'plugin_admin_add_page') );

	}
	
	/*
		FILTER HOOK for comment_form_logged_in
		
		PURPOSE:
		Wordpress's comment_form() suppresses name/email/website input fields for logged-in users, on the assumption that profile data
		will be used instead.  Hook the comment_form_logged_in filter and inject these fields back into the HTML page response.
		
		ASSUMES:
		There is a 'guest_comments_enabled' setting in user metadata.  If present and equal to 'on' for the currently logged-in user
		then this filter acts.  If not then this filter does nothing.
		
	*/
	function my_comment_form_logged_in( $logged_in_as, $commenter, $user_identity )
	{
		global $current_user;
		get_currentuserinfo();
		$guest_comments_enabled = get_user_meta( $current_user->ID, "guest_comments_enabled", true );

		$hide_url = $this->get_hide_url();
		
		if ( is_user_logged_in() && $guest_comments_enabled )
		{	
			$req = get_option( 'require_name_email' );
		
			$form = '<p class="comment-form-author">';
			$form .=  '<input id="author" name="author" type="text" class="txt" tabindex="1" value="' . esc_attr( $commenter['comment_author'] ) . '" size="30" />';
			$form .=  '<label for="author">Name ' . ( $req ? '<span class="required">(required)</span>' : '' ) . '</label>';
			$form .= '</p>';
		
			$form .= '<p class="comment-form-email">';
			$form .=  '<input id="email" name="email" type="text" class="txt" tabindex="2" value="' . esc_attr( $commenter['comment_author_email'] ) . '" size="30" ' . ( $req ? 'aria-required="true"' : '' ) . ' />';
			$form .=  '<label for="email">Email (will not be published) ' . ( $req ? '<span class="required">(required)</span>' : '' ) . '</label>';
			$form .= '</p>';

			if ( !$hide_url )
			{
				$form .= '<p class="comment-form-url">';
				$form .=  '<input id="url" name="url" type="text" class="txt" tabindex="3" value="' . esc_attr( $commenter['comment_author_url'] ) . '" size="30" />';
				$form .=  '<label for="url">Website</label>';
				$form .= '</p>';
			}
					
			$logged_in_as = $form;
		}
		
		return $logged_in_as;
	
	}
	
	/*
		FILTER HOOK for preprocess_comment
	
		PURPOSE:
		Allow multiple users to share a single login, yet still attach their individual name/email/website to comments.
		
		METHOD:
		Wordpress's wp-comments-post.php reads POSTed name/email/website data, but then for logged-in users it replaces that data
		with profile data.  For cases where multiple users share a single login that isn't appropriate, so this filter hook checks
		whether POSTed data exists and if so, replaces the profile data in $commentdata.
		
		ASSUMES:
		There is a 'guest_comments_enabled' setting in user metadata.  If present and equal to 'on' for the currently logged-in user
		then this filter acts.  If not then this filter does nothing.		
	*/
	function my_preprocess_comment ( $commentdata )
	{	
		global $current_user;
		get_currentuserinfo();
		$guest_comments_enabled = get_user_meta( $current_user->ID, "guest_comments_enabled", true );

		$hide_url = $this->get_hide_url();
		
		if ( is_user_logged_in() && $guest_comments_enabled )
		{
			$commentdata[ 'comment_author' ]        = ( isset($_POST['author']) ) ? trim(strip_tags($_POST['author'])) : $commentdata['comment_author'];
			$commentdata[ 'comment_author_email' ]  = ( isset($_POST['email']) )  ? trim($_POST['email'])              : $commentdata['comment_author_email'];
			if ( !$hide_url )
			{
				$commentdata[ 'comment_author_url'] = ( isset($_POST['url']) )    ? trim($_POST['url'])                : $commentdata['comment_author_url'];
			}
		}
		
		return $commentdata;
	}
	
	/*
		ACTION HOOK handles 'show_user_profile' and 'edit_user_profile'
		
		PURPOSE
		Render extra field on user profile page to expose guest_comments_enabled yes/no setting.

		Editing this setting is restricted to users with 'edit_user' capability for all users.  This is usually only Administrators.
	*/
	function render_user_profile_setting( $user )
	{ 
		$guest_comments_enabled = get_user_meta( $user->ID, "guest_comments_enabled", true );
		$may_edit = current_user_can( 'edit_user' );

		echo '
	    <h3>Guest Comments Plugin</h3>
	    <table class="form-table">
		    <tr>
			    <th>
			    	<label for="guest-comments-enabled">Comment as guest?</label>
			    </th>
			    <td>
				    <input type="checkbox" name="guest_comments_enabled" id="guest_comments_enabled" '
				    . ( $guest_comments_enabled ? "checked=\"checked\"" : "")
				    . ( $may_edit ? "" : "disabled=\"disabled\"")
				    . '"/>
				    <span class="description">&nbsp;&nbsp; If multiple people share this account, turn this checkbox on so each person can enter their own name/email/website when they comment on posts or pages.  
				    <span style="color:red;">This setting can only be altered by Administrators.</span></span>
			    </td>
		    </tr>
	    </table>';
	}
	
	/*
		ACTION HOOK handles 'personal_options_update' and 'edit_user_profile_update'
		
		PURPOSE
		Save extra user profile field guest_comments_enabled (yes/no setting) to user metadata to it is available later on

		Editing this setting is restricted to users with 'edit_user' capability for all users.  This is usually only Administrators.
	*/
	function save_user_profile_setting( $user_id )
	{
		// Bail if current user doesn't have the 'edit_user' capability
	    if ( !current_user_can( 'edit_user' ) ) { return false; }
	    
	    $guest_comments_enabled = ( isset( $_POST['guest_comments_enabled'] ) );
	    update_user_meta( $user_id, 'guest_comments_enabled', $guest_comments_enabled );
	}
	
	/*
		ACTION HOOK handles 'comment_form_default_fields'
		
		PURPOSE:
		Simplify and reduce spam by removing the unnecessary author URL field
	*/
	function remove_url_field( $fields )
	{
		$hide_url = $this->get_hide_url();
		
		if ( $hide_url && isset( $fields['url'] ) )
			 unset( $fields['url'] );
			 
		return $fields;
	}

/* ----------------- PLUGIN SETTINGS ---------------------*/

	protected function get_hide_url()
	{
		$options = get_option('guest-comments-options');
		$hide_url = isset( $options['guest-comments-disable-url'] ) ;
		return $hide_url;
	}
	
	// add the admin settings and such (handles 'admin_init' action)
	function plugin_admin_init()
	{
		register_setting( 'guest-comments-options', 'guest-comments-options', array( $this, 'validate_plugin_options' ) );
		add_settings_section( 'guest-comments-section', '', array( $this, 'render_guest_comments_section' ), 'guest-comments-page' );
		add_settings_field( 'guest-comments-disable-url', 'Comment fields', array( $this, 'render_guest_comments_disable_url' ), 'guest-comments-page', 'guest-comments-section' );
	}
	
	// Add the plugin options menu (handles 'admin_menu' action)
	function plugin_admin_add_page()
	{
		add_options_page( 'Guest-Comments', 'Guest Comments', 'manage_options', 'guest-comments-page', array( $this, 'render_guest_comments_page' ) );
	}

	// Validation fucntion
	function validate_plugin_options( $input )
	{
		return $input;	// always valid, ie: no validation is done.
	}

	// Render section header
	function render_guest_comments_section()
	{
		// No section header needed, so do nothing
	}
	
	// Display the plugin options page
	function render_guest_comments_page()
	{
		echo '
			<div class="wrap">
				<h2>Guest Comments Plugin Settings</h2>
				<form action="options.php" method="post">';
				
					settings_fields('guest-comments-options');
					do_settings_sections('guest-comments-page');
					
					echo ' 
					<p class="submit"><input type="submit" name="submit" id="submit" class="button-primary" value="Save Changes"  /></p>
				</form>
			</div>';
	}

	// Render the guest-comments-diable-url setting
	function render_guest_comments_disable_url()
	{
		$hide_url = $this->get_hide_url();

		echo '<input type=checkbox id="guest-comments-disable-url" 
					 name="guest-comments-options[guest-comments-disable-url]" '
					 . ( $hide_url ? 'checked=\"checked\"' : '' ) . '
			  />
			  <label for="guest-comments-disable-url">&nbsp; Hide the website field on comment forms</label>';
	}
	
}

// Initialize the plugin
$myGuestComments = new guest_comments();
$myGuestComments->init();

?>